"use strict";
/**
 * @fileOverview spread sheet api
 * @name api
 * @author Yuhei Aihara <yu.e.yu.4119@gmail.com>
 * https://github.com/iyu/spread2json
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const lodash_1 = __importDefault(require("lodash"));
const googleapis_1 = require("googleapis");
/**
 * @see https://developers.google.com/sheets/guides/authorizing
 */
const SCOPE = [
    'https://www.googleapis.com/auth/spreadsheets',
    'https://www.googleapis.com/auth/drive',
];
class SpreadSheetApi {
    constructor() {
        this.opts = {
            client_id: undefined,
            client_secret: undefined,
            redirect_url: 'http://localhost',
            token_file: {
                use: true,
                path: './dist/token.json',
            },
        };
    }
    /**
     * setup
     * @param {Object} options
     * @param {string} options.client_id
     * @param {string} options.client_secret
     * @param {string} [options.redirect_url='http://localhost']
     * @param {Object} [options.token_file]
     * @param {boolean} [options.token_file.use=true]
     * @param {string} [options.token_file.path='./dist/token.json']
     * @example
     * options = {
     *   client_id: 'xxx',
     *   client_secret: 'xxx',
     *   redirect_url: 'http://localhost',
     *   token_file: {
     *     use: true
     *     path: './token.json'
     *   }
     * }
     */
    setup(options) {
        const opts = lodash_1.default.assign(this.opts, options);
        this.oAuth2 = new googleapis_1.google.auth.OAuth2(opts.client_id, opts.client_secret, opts.redirect_url);
        if (!this.opts.token_file || !this.opts.token_file.path) {
            this.opts.token_file = {
                use: false,
                path: './dist/token.json',
            };
        }
        if (!this.opts.token_file.use) {
            return this;
        }
        const tokenDir = path_1.default.dirname(this.opts.token_file.path);
        if (!fs_1.default.existsSync(tokenDir)) {
            fs_1.default.mkdirSync(tokenDir);
        }
        if (fs_1.default.existsSync(this.opts.token_file.path)) {
            const token = JSON.parse(fs_1.default.readFileSync(this.opts.token_file.path, 'utf8'));
            this.oAuth2.setCredentials(token);
        }
        return this;
    }
    /**
     * generate auth url
     * @see https://github.com/google/google-api-nodejs-client/#generating-an-authentication-url
     * @param {Object} [opts={}]
     * @param {string|Array} [opts.scope=SCOPE]
     * @return {string|Error} - URL to consent page.
     */
    generateAuthUrl(opts) {
        if (!this.oAuth2) {
            return new Error('not been setup yet.');
        }
        opts = opts || {};
        opts.scope = opts.scope || SCOPE;
        return this.oAuth2.generateAuthUrl.call(this.oAuth2, opts);
    }
    /**
     * get access token
     * @param {string} code
     */
    async getAccessToken(code) {
        if (!this.oAuth2) {
            throw new Error('not been setup yet.');
        }
        let result;
        try {
            ({ tokens: result } = await this.oAuth2.getToken(code));
        }
        catch (e) {
            e.status = 401;
            throw e;
        }
        if (this.opts.token_file.use && this.oAuth2 && result) {
            this.oAuth2.setCredentials(result);
            fs_1.default.writeFileSync(this.opts.token_file.path, JSON.stringify(result));
        }
        return result;
    }
    /**
     * refresh access token
     * @param {Credentials} [credentials] - oAuth2.getToken result
     */
    async refreshAccessToken(credentials) {
        if (!this.oAuth2) {
            throw new Error('not been setup yet.');
        }
        if (credentials) {
            this.oAuth2.setCredentials(credentials);
        }
        let result;
        try {
            ({ credentials: result } = await this.oAuth2.refreshAccessToken());
        }
        catch (e) {
            e.status = 401;
            throw e;
        }
        if (this.opts.token_file.use) {
            this.oAuth2.setCredentials(result);
            fs_1.default.writeFileSync(this.opts.token_file.path, JSON.stringify(result));
        }
        return result;
    }
    /**
     * get spreadsheet
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {Object} [opts] - drive options @see https://developers.google.com/drive/v3/reference/files/list
     * @param {string} [opts.corpus]
     * @param {string} [opts.orderBy]
     * @param {integer} [opts.pageSize]
     * @param {string} [opts.pageToken]
     * @param {string} [opts.q]
     * @param {string} [opts.spaces]
     * @param {Function} callback
     */
    async getSpreadsheet(credentials, opts) {
        if (!this.oAuth2) {
            throw new Error('not been setup yet.');
        }
        if (credentials) {
            this.oAuth2.setCredentials(credentials);
        }
        const drive = googleapis_1.google.drive('v3');
        const result = await drive.files.list(lodash_1.default.assign({
            auth: this.oAuth2,
            q: 'mimeType=\'application/vnd.google-apps.spreadsheet\'',
        }, opts));
        return result.data.files;
    }
    /**
     * get worksheet
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {string} spreadsheetId
     * @param {Function} callback
     */
    async getWorksheet(credentials, spreadsheetId) {
        if (!this.oAuth2) {
            throw new Error('not been setup yet.');
        }
        if (credentials) {
            this.oAuth2.setCredentials(credentials);
        }
        const sheets = googleapis_1.google.sheets('v4');
        const result = await sheets.spreadsheets.get({
            auth: this.oAuth2,
            spreadsheetId,
        });
        return result.data;
    }
    /**
     * add worksheet
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {string} spreadsheetId
     * @param {string} title - worksheet title
     * @param {number} rowCount
     * @param {number} columnCount
     */
    async addWorksheet(credentials, spreadsheetId, title, rowCount, columnCount) {
        if (!this.oAuth2) {
            throw new Error('not been setup yet.');
        }
        if (credentials) {
            this.oAuth2.setCredentials(credentials);
        }
        const sheets = googleapis_1.google.sheets('v4');
        const result = await sheets.spreadsheets.batchUpdate({
            auth: this.oAuth2,
            spreadsheetId,
            requestBody: {
                requests: [
                    {
                        addSheet: {
                            properties: {
                                title,
                                gridProperties: {
                                    rowCount,
                                    columnCount,
                                    frozenRowCount: 3,
                                    frozenColumnCount: 1,
                                },
                                tabColor: {
                                    red: 1,
                                    green: 0,
                                    blue: 0,
                                    alpha: 1,
                                },
                            },
                        },
                    },
                ],
            },
        });
        return result.data;
    }
    /**
     * get list raw
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {string} spreadsheetId
     * @param {string} sheetName
     */
    async getList(credentials, spreadsheetId, sheetName) {
        if (!this.oAuth2) {
            throw new Error('not been setup yet.');
        }
        if (credentials) {
            this.oAuth2.setCredentials(credentials);
        }
        const sheets = googleapis_1.google.sheets('v4');
        const result = await sheets.spreadsheets.values.get({
            auth: this.oAuth2,
            spreadsheetId,
            range: sheetName,
            majorDimension: 'ROWS',
            valueRenderOption: 'FORMATTED_VALUE',
            dateTimeRenderOption: 'FORMATTED_STRING',
        });
        return result.data;
    }
    /**
     * get cells raw
     * @param {Credentials} [credentials] - oAuth2.getToken result
     * @param {string} spreadsheetId
     * @param {string} sheetName
     * @param {Array[]} entry
     */
    async batchCells(credentials, spreadsheetId, sheetName, entry) {
        if (!this.oAuth2) {
            throw new Error('not been setup yet.');
        }
        if (credentials) {
            this.oAuth2.setCredentials(credentials);
        }
        const sheets = googleapis_1.google.sheets('v4');
        const result = await sheets.spreadsheets.values.batchUpdate({
            auth: this.oAuth2,
            spreadsheetId,
            requestBody: {
                valueInputOption: 'RAW',
                data: [
                    {
                        range: sheetName,
                        values: entry,
                    },
                ],
            },
        });
        return result.data;
    }
}
exports.default = new SpreadSheetApi();
